"""Gauss intergration rules."""

# This file is based on the Hyena file gausstria.hpp,
# which was released under the MIT License for the Bempp project.
# The original license message is below

# Copyright (C) 2009-2010 Matthias Messner, Michael Messner, Franz
# Rammerstorfer, Peter Urthaler
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# Note: This file is originally part of the HyENA project
# (http:#portal.tugraz.at/portal/page/portal/Files/i2610/files/Forschung/Software/HyENA/html/index.html)
# and has been relicensed with permission by the HyENA authors. This does not
# affect the license of any
# other part of HyENA.

import numpy as _np

highest_order = 20

# Indices after which the points and weights can be found in the
# arrays for a given order of the rule from 1 to 79.
#
points_address = _np.array(
    [
        # negative numbers indicate that this rule does not exist
        # number of points
        # 1   2     3     4     5     6     7     8     9    10
        0,
        -2,
        1,
        4,
        -5,
        8,
        14,
        -8,
        21,
        -11,
        #
        # 11   12    13    14    15    16    17    18    19    20
        -11,
        30,
        42,
        -14,
        55,
        70,
        -18,
        -18,
        86,
        -21,
        #
        # 21   22    23    24    25    26    27    28    29    30
        105,
        -24,
        -24,
        -24,
        126,
        -27,
        151,
        -32,
        -32,
        -32,
        #
        # 31   32    33    34    35    36    37    38    39    40
        -32,
        -32,
        178,
        -36,
        -36,
        -36,
        211,
        -41,
        -41,
        -41,
        #
        # 41   42    43    44    45    46    47    48    49    50
        -41,
        248,
        -47,
        -47,
        -47,
        -47,
        -47,
        290,
        -51,
        -51,
        #
        # 51   52    53    54    55    56    57    58    59    60
        -51,
        338,
        -60,
        -60,
        -60,
        -60,
        -60,
        -60,
        -60,
        -60,
        #
        # 61   62    63    64    65    66    67    68    69    70
        390,
        -69,
        -69,
        -69,
        -69,
        -69,
        -69,
        -69,
        -69,
        451,
        #
        # 71   72    73    74    75    76    77    78    79
        -72,
        -72,
        521,
        -78,
        -78,
        -78,
        -78,
        -78,
        594,
    ]
)

# give number of points for each order
points_per_order = _np.array(
    [
        # order of complete polynomial:
        # 1  2   3   4   5   6   7   8   9  10  11  12  13  14
        1,
        3,
        4,
        6,
        7,
        12,
        13,
        16,
        19,
        25,
        27,
        33,
        37,
        42,
        # 15 16 17  18  19  20
        48,
        52,
        61,
        70,
        73,
        79,
    ]
)

#
#  Gauss points for symmetric quadrature rules on triangles. Coordinates are
#  given in barycentric coordinates.
#
coords = _np.array(
    [
        # Coordinates for number 1
        0.3333333333333330,
        0.3333333333333330,
        0.3333333333333330,
        # Coordinates for number 3
        0.6666666666666670,
        0.1666666666666670,
        0.1666666666666660,
        0.1666666666666670,
        0.6666666666666670,
        0.1666666666666660,
        0.1666666666666670,
        0.1666666666666670,
        0.6666666666666661,
        # Coordinates for number 4
        0.3333333333333330,
        0.3333333333333330,
        0.3333333333333340,
        0.6000000000000000,
        0.2000000000000000,
        0.2000000000000000,
        0.2000000000000000,
        0.6000000000000000,
        0.2000000000000000,
        0.2000000000000000,
        0.2000000000000000,
        0.6000000000000000,
        # Coordinates for number 6
        0.1081030181680700,
        0.4459484909159651,
        0.4459484909159651,
        0.8168475729804590,
        0.0915762135097710,
        0.0915762135097700,
        0.4459484909159651,
        0.1081030181680700,
        0.4459484909159651,
        0.4459484909159651,
        0.4459484909159651,
        0.1081030181680700,
        0.0915762135097710,
        0.8168475729804590,
        0.0915762135097700,
        0.0915762135097710,
        0.0915762135097710,
        0.8168475729804580,
        # Coordinates for number 7
        0.3333333333333330,
        0.3333333333333330,
        0.3333333333333340,
        0.0597158717897700,
        0.4701420641051150,
        0.4701420641051150,
        0.7974269853530870,
        0.1012865073234560,
        0.1012865073234570,
        0.4701420641051150,
        0.0597158717897700,
        0.4701420641051150,
        0.4701420641051150,
        0.4701420641051150,
        0.0597158717897700,
        0.1012865073234560,
        0.7974269853530870,
        0.1012865073234570,
        0.1012865073234560,
        0.1012865073234560,
        0.7974269853530880,
        # Coordinates for number 9
        0.4375252483833840,
        0.4375252483833840,
        0.1249495032332320,
        0.4375252483833840,
        0.1249495032332320,
        0.4375252483833840,
        0.1249495032332320,
        0.4375252483833840,
        0.4375252483833840,
        0.7971126518600710,
        0.1654099273898410,
        0.0374774207500880,
        0.7971126518600710,
        0.0374774207500880,
        0.1654099273898410,
        0.1654099273898410,
        0.7971126518600710,
        0.0374774207500880,
        0.0374774207500880,
        0.7971126518600710,
        0.1654099273898410,
        0.0374774207500880,
        0.1654099273898410,
        0.7971126518600710,
        0.1654099273898410,
        0.0374774207500880,
        0.7971126518600710,
        # Coordinates for number 12
        0.5014265096581790,
        0.2492867451709100,
        0.2492867451709110,
        0.8738219710169960,
        0.0630890144915020,
        0.0630890144915020,
        0.0531450498448170,
        0.3103524510337841,
        0.6365024991213991,
        0.2492867451709100,
        0.5014265096581790,
        0.2492867451709110,
        0.2492867451709100,
        0.2492867451709100,
        0.5014265096581800,
        0.0630890144915020,
        0.8738219710169960,
        0.0630890144915020,
        0.0630890144915020,
        0.0630890144915020,
        0.8738219710169960,
        0.0531450498448170,
        0.6365024991213991,
        0.3103524510337841,
        0.3103524510337841,
        0.6365024991213991,
        0.0531450498448170,
        0.6365024991213991,
        0.0531450498448170,
        0.3103524510337841,
        0.3103524510337841,
        0.0531450498448170,
        0.6365024991213991,
        0.6365024991213991,
        0.3103524510337841,
        0.0531450498448170,
        # Coordinates for number 13
        0.3333333333333330,
        0.3333333333333330,
        0.3333333333333330,
        0.4793080678419200,
        0.2603459660790400,
        0.2603459660790401,
        0.8697397941955681,
        0.0651301029022160,
        0.0651301029022160,
        0.0486903154253160,
        0.3128654960048741,
        0.6384441885698101,
        0.2603459660790400,
        0.4793080678419200,
        0.2603459660790401,
        0.2603459660790400,
        0.2603459660790400,
        0.4793080678419201,
        0.0651301029022160,
        0.8697397941955681,
        0.0651301029022160,
        0.0651301029022160,
        0.0651301029022160,
        0.8697397941955681,
        0.0486903154253160,
        0.6384441885698101,
        0.3128654960048741,
        0.3128654960048741,
        0.6384441885698101,
        0.0486903154253160,
        0.6384441885698101,
        0.0486903154253160,
        0.3128654960048741,
        0.3128654960048741,
        0.0486903154253160,
        0.6384441885698101,
        0.6384441885698101,
        0.3128654960048741,
        0.0486903154253160,
        # Coordinates for number 15
        0.0649305131591650,
        0.0649305131591650,
        0.8701389736816700,
        0.0649305131591650,
        0.8701389736816700,
        0.0649305131591650,
        0.8701389736816700,
        0.0649305131591650,
        0.0649305131591650,
        0.1983844766815020,
        0.5170399390693250,
        0.2845755842491730,
        0.1983844766815020,
        0.2845755842491730,
        0.5170399390693250,
        0.2845755842491730,
        0.1983844766815020,
        0.5170399390693250,
        0.5170399390693250,
        0.1983844766815020,
        0.2845755842491730,
        0.5170399390693250,
        0.2845755842491730,
        0.1983844766815020,
        0.2845755842491730,
        0.5170399390693250,
        0.1983844766815020,
        0.6425773438226970,
        0.0438634717923710,
        0.3135591843849320,
        0.6425773438226970,
        0.3135591843849320,
        0.0438634717923710,
        0.3135591843849320,
        0.6425773438226970,
        0.0438634717923710,
        0.0438634717923710,
        0.6425773438226970,
        0.3135591843849320,
        0.0438634717923710,
        0.3135591843849320,
        0.6425773438226970,
        0.3135591843849320,
        0.0438634717923710,
        0.6425773438226970,
        # Coordinates for number 16
        0.3333333333333330,
        0.3333333333333330,
        0.3333333333333330,
        0.0814148234145540,
        0.4592925882927231,
        0.4592925882927231,
        0.6588613844964801,
        0.1705693077517600,
        0.1705693077517600,
        0.8989055433659380,
        0.0505472283170310,
        0.0505472283170310,
        0.0083947774099580,
        0.2631128296346380,
        0.7284923929554041,
        0.4592925882927231,
        0.0814148234145540,
        0.4592925882927231,
        0.4592925882927231,
        0.4592925882927231,
        0.0814148234145540,
        0.1705693077517600,
        0.6588613844964801,
        0.1705693077517600,
        0.1705693077517600,
        0.1705693077517600,
        0.6588613844964801,
        0.0505472283170310,
        0.8989055433659380,
        0.0505472283170310,
        0.0505472283170310,
        0.0505472283170310,
        0.8989055433659380,
        0.0083947774099580,
        0.7284923929554041,
        0.2631128296346380,
        0.2631128296346380,
        0.7284923929554041,
        0.0083947774099580,
        0.7284923929554041,
        0.0083947774099580,
        0.2631128296346380,
        0.2631128296346380,
        0.0083947774099580,
        0.7284923929554041,
        0.7284923929554041,
        0.2631128296346380,
        0.0083947774099580,
        # Coordinates for number 19
        0.3333333333333329,
        0.3333333333333329,
        0.3333333333333340,
        0.0206349616025250,
        0.4896825191987380,
        0.4896825191987370,
        0.1258208170141270,
        0.4370895914929371,
        0.4370895914929360,
        0.6235929287619349,
        0.1882035356190330,
        0.1882035356190321,
        0.9105409732110950,
        0.0447295133944530,
        0.0447295133944521,
        0.0368384120547360,
        0.2219629891607660,
        0.7411985987844981,
        0.4896825191987380,
        0.0206349616025250,
        0.4896825191987370,
        0.4896825191987380,
        0.4896825191987380,
        0.0206349616025240,
        0.4370895914929371,
        0.1258208170141270,
        0.4370895914929360,
        0.4370895914929371,
        0.4370895914929371,
        0.1258208170141260,
        0.1882035356190330,
        0.6235929287619350,
        0.1882035356190321,
        0.1882035356190330,
        0.1882035356190330,
        0.6235929287619340,
        0.0447295133944530,
        0.9105409732110950,
        0.0447295133944521,
        0.0447295133944530,
        0.0447295133944530,
        0.9105409732110941,
        0.0368384120547360,
        0.7411985987844981,
        0.2219629891607660,
        0.2219629891607660,
        0.7411985987844981,
        0.0368384120547360,
        0.7411985987844981,
        0.0368384120547360,
        0.2219629891607660,
        0.2219629891607660,
        0.0368384120547360,
        0.7411985987844981,
        0.7411985987844981,
        0.2219629891607660,
        0.0368384120547360,
        # Coordinates for number 21
        0.0369603304333780,
        0.4815198347833110,
        0.4815198347833110,
        0.4815198347833110,
        0.0369603304333780,
        0.4815198347833110,
        0.4815198347833110,
        0.4815198347833110,
        0.0369603304333780,
        0.1927920403641200,
        0.4036039798179400,
        0.4036039798179400,
        0.4036039798179400,
        0.1927920403641200,
        0.4036039798179400,
        0.4036039798179400,
        0.4036039798179400,
        0.1927920403641200,
        0.9096219804312460,
        0.0451890097843770,
        0.0451890097843770,
        0.0451890097843770,
        0.9096219804312460,
        0.0451890097843770,
        0.0451890097843770,
        0.0451890097843770,
        0.9096219804312460,
        0.1369912012649040,
        0.2182900709713810,
        0.6447187277637150,
        0.1369912012649040,
        0.6447187277637150,
        0.2182900709713810,
        0.2182900709713810,
        0.1369912012649040,
        0.6447187277637150,
        0.2182900709713810,
        0.6447187277637150,
        0.1369912012649040,
        0.6447187277637150,
        0.1369912012649040,
        0.2182900709713810,
        0.6447187277637150,
        0.2182900709713810,
        0.1369912012649040,
        0.0304243617288200,
        0.2220631655373180,
        0.7475124727338620,
        0.0304243617288200,
        0.7475124727338620,
        0.2220631655373180,
        0.2220631655373180,
        0.0304243617288200,
        0.7475124727338620,
        0.2220631655373180,
        0.7475124727338620,
        0.0304243617288200,
        0.7475124727338620,
        0.0304243617288200,
        0.2220631655373180,
        0.7475124727338620,
        0.2220631655373180,
        0.0304243617288200,
        # Coordinates for number 25
        0.3333333333333329,
        0.3333333333333329,
        0.3333333333333340,
        0.0288447332326850,
        0.4855776333836570,
        0.4855776333836580,
        0.7810368490299259,
        0.1094815754850370,
        0.1094815754850370,
        0.1417072194148800,
        0.3079398387641210,
        0.5503529418209990,
        0.0250035347626860,
        0.2466725606399030,
        0.7283239045974110,
        0.0095408154002990,
        0.0668032510122000,
        0.9236559335875009,
        0.4855776333836570,
        0.0288447332326850,
        0.4855776333836580,
        0.4855776333836570,
        0.4855776333836570,
        0.0288447332326860,
        0.1094815754850370,
        0.7810368490299259,
        0.1094815754850370,
        0.1094815754850370,
        0.1094815754850370,
        0.7810368490299260,
        0.1417072194148800,
        0.5503529418209990,
        0.3079398387641210,
        0.3079398387641210,
        0.5503529418209990,
        0.1417072194148800,
        0.5503529418209990,
        0.1417072194148800,
        0.3079398387641210,
        0.3079398387641210,
        0.1417072194148800,
        0.5503529418209990,
        0.5503529418209990,
        0.3079398387641210,
        0.1417072194148800,
        0.0250035347626860,
        0.7283239045974110,
        0.2466725606399030,
        0.2466725606399030,
        0.7283239045974110,
        0.0250035347626860,
        0.7283239045974110,
        0.0250035347626860,
        0.2466725606399030,
        0.2466725606399030,
        0.0250035347626860,
        0.7283239045974110,
        0.7283239045974110,
        0.2466725606399030,
        0.0250035347626860,
        0.0095408154002990,
        0.9236559335875000,
        0.0668032510122010,
        0.0668032510122000,
        0.9236559335875000,
        0.0095408154003000,
        0.9236559335875000,
        0.0095408154002990,
        0.0668032510122010,
        0.0668032510122000,
        0.0095408154002990,
        0.9236559335875010,
        0.9236559335875000,
        0.0668032510122000,
        0.0095408154003000,
        # Coordinates for number 27
        -0.0692220965415170,
        0.5346110482707580,
        0.5346110482707590,
        0.2020613940682900,
        0.3989693029658549,
        0.3989693029658550,
        0.5933801991374351,
        0.2033099004312820,
        0.2033099004312830,
        0.7612981754348371,
        0.1193509122825810,
        0.1193509122825820,
        0.9352701037774480,
        0.0323649481112760,
        0.0323649481112760,
        0.0501781383104950,
        0.3566206482612930,
        0.5932012134282119,
        0.0210220165361660,
        0.1714889803040420,
        0.8074890031597919,
        0.5346110482707580,
        -0.0692220965415170,
        0.5346110482707590,
        0.5346110482707580,
        0.5346110482707580,
        -0.0692220965415160,
        0.3989693029658549,
        0.2020613940682900,
        0.3989693029658550,
        0.3989693029658549,
        0.3989693029658549,
        0.2020613940682900,
        0.2033099004312820,
        0.5933801991374351,
        0.2033099004312830,
        0.2033099004312820,
        0.2033099004312820,
        0.5933801991374361,
        0.1193509122825810,
        0.7612981754348371,
        0.1193509122825820,
        0.1193509122825810,
        0.1193509122825810,
        0.7612981754348381,
        0.0323649481112760,
        0.9352701037774480,
        0.0323649481112760,
        0.0323649481112760,
        0.0323649481112760,
        0.9352701037774480,
        0.0501781383104950,
        0.5932012134282129,
        0.3566206482612920,
        0.3566206482612930,
        0.5932012134282129,
        0.0501781383104940,
        0.5932012134282129,
        0.0501781383104950,
        0.3566206482612920,
        0.3566206482612930,
        0.0501781383104950,
        0.5932012134282119,
        0.5932012134282129,
        0.3566206482612930,
        0.0501781383104940,
        0.0210220165361660,
        0.8074890031597919,
        0.1714889803040420,
        0.1714889803040420,
        0.8074890031597919,
        0.0210220165361660,
        0.8074890031597919,
        0.0210220165361660,
        0.1714889803040420,
        0.1714889803040420,
        0.0210220165361660,
        0.8074890031597919,
        0.8074890031597919,
        0.1714889803040420,
        0.0210220165361660,
        # Coordinates for number 33
        0.0235652204523900,
        0.4882173897738051,
        0.4882173897738050,
        0.1205512154110790,
        0.4397243922944600,
        0.4397243922944611,
        0.4575792299757679,
        0.2712103850121160,
        0.2712103850121161,
        0.7448477089168280,
        0.1275761455415860,
        0.1275761455415859,
        0.9573652990935791,
        0.0213173504532100,
        0.0213173504532110,
        0.1153434945346980,
        0.2757132696855141,
        0.6089432357797879,
        0.0228383322222570,
        0.2813255809899400,
        0.6958360867878031,
        0.0257340505483300,
        0.1162519159075970,
        0.8580140335440730,
        0.4882173897738051,
        0.0235652204523900,
        0.4882173897738050,
        0.4882173897738051,
        0.4882173897738051,
        0.0235652204523900,
        0.4397243922944600,
        0.1205512154110790,
        0.4397243922944611,
        0.4397243922944600,
        0.4397243922944600,
        0.1205512154110800,
        0.2712103850121160,
        0.4575792299757679,
        0.2712103850121161,
        0.2712103850121160,
        0.2712103850121160,
        0.4575792299757680,
        0.1275761455415860,
        0.7448477089168280,
        0.1275761455415859,
        0.1275761455415860,
        0.1275761455415860,
        0.7448477089168279,
        0.0213173504532100,
        0.9573652990935791,
        0.0213173504532110,
        0.0213173504532100,
        0.0213173504532100,
        0.9573652990935801,
        0.1153434945346980,
        0.6089432357797880,
        0.2757132696855140,
        0.2757132696855141,
        0.6089432357797880,
        0.1153434945346979,
        0.6089432357797880,
        0.1153434945346980,
        0.2757132696855140,
        0.2757132696855141,
        0.1153434945346980,
        0.6089432357797879,
        0.6089432357797880,
        0.2757132696855141,
        0.1153434945346979,
        0.0228383322222570,
        0.6958360867878030,
        0.2813255809899401,
        0.2813255809899400,
        0.6958360867878030,
        0.0228383322222571,
        0.6958360867878030,
        0.0228383322222570,
        0.2813255809899401,
        0.2813255809899400,
        0.0228383322222570,
        0.6958360867878031,
        0.6958360867878030,
        0.2813255809899400,
        0.0228383322222571,
        0.0257340505483300,
        0.8580140335440730,
        0.1162519159075970,
        0.1162519159075970,
        0.8580140335440730,
        0.0257340505483300,
        0.8580140335440730,
        0.0257340505483300,
        0.1162519159075970,
        0.1162519159075970,
        0.0257340505483300,
        0.8580140335440730,
        0.8580140335440730,
        0.1162519159075970,
        0.0257340505483300,
        # Coordinates for number 37
        0.3333333333333329,
        0.3333333333333329,
        0.3333333333333340,
        0.0099036301205910,
        0.4950481849397050,
        0.4950481849397041,
        0.0625667297808520,
        0.4687166351095740,
        0.4687166351095740,
        0.1709573263974470,
        0.4145213368012770,
        0.4145213368012760,
        0.5412008559143370,
        0.2293995720428310,
        0.2293995720428320,
        0.7711510096073399,
        0.1144244951963300,
        0.1144244951963301,
        0.9503772172730819,
        0.0248113913634590,
        0.0248113913634590,
        0.0948538283795790,
        0.2687949970587610,
        0.6363511745616601,
        0.0181007732788070,
        0.2917300667342880,
        0.6901691599869050,
        0.0222330766740900,
        0.1263573854916690,
        0.8514095378342410,
        0.4950481849397050,
        0.0099036301205910,
        0.4950481849397041,
        0.4950481849397050,
        0.4950481849397050,
        0.0099036301205900,
        0.4687166351095740,
        0.0625667297808520,
        0.4687166351095740,
        0.4687166351095740,
        0.4687166351095740,
        0.0625667297808520,
        0.4145213368012770,
        0.1709573263974470,
        0.4145213368012760,
        0.4145213368012770,
        0.4145213368012770,
        0.1709573263974460,
        0.2293995720428310,
        0.5412008559143370,
        0.2293995720428320,
        0.2293995720428310,
        0.2293995720428310,
        0.5412008559143380,
        0.1144244951963300,
        0.7711510096073399,
        0.1144244951963301,
        0.1144244951963300,
        0.1144244951963300,
        0.7711510096073399,
        0.0248113913634590,
        0.9503772172730819,
        0.0248113913634590,
        0.0248113913634590,
        0.0248113913634590,
        0.9503772172730819,
        0.0948538283795790,
        0.6363511745616601,
        0.2687949970587610,
        0.2687949970587610,
        0.6363511745616601,
        0.0948538283795790,
        0.6363511745616601,
        0.0948538283795790,
        0.2687949970587610,
        0.2687949970587610,
        0.0948538283795790,
        0.6363511745616601,
        0.6363511745616601,
        0.2687949970587610,
        0.0948538283795790,
        0.0181007732788070,
        0.6901691599869050,
        0.2917300667342881,
        0.2917300667342880,
        0.6901691599869050,
        0.0181007732788070,
        0.6901691599869050,
        0.0181007732788070,
        0.2917300667342881,
        0.2917300667342880,
        0.0181007732788070,
        0.6901691599869050,
        0.6901691599869050,
        0.2917300667342880,
        0.0181007732788070,
        0.0222330766740900,
        0.8514095378342410,
        0.1263573854916690,
        0.1263573854916690,
        0.8514095378342410,
        0.0222330766740900,
        0.8514095378342410,
        0.0222330766740900,
        0.1263573854916690,
        0.1263573854916690,
        0.0222330766740900,
        0.8514095378342410,
        0.8514095378342410,
        0.1263573854916690,
        0.0222330766740900,
        # Coordinates for number 42
        0.0220721792756430,
        0.4889639103621790,
        0.4889639103621780,
        0.1647105613190920,
        0.4176447193404539,
        0.4176447193404539,
        0.4530449433823229,
        0.2734775283088389,
        0.2734775283088380,
        0.6455889351749130,
        0.1772055324125430,
        0.1772055324125440,
        0.8764002338182550,
        0.0617998830908730,
        0.0617998830908719,
        0.9612180775025981,
        0.0193909612487010,
        0.0193909612487010,
        0.0571247574036480,
        0.1722666878213560,
        0.7706085547749961,
        0.0929162493569720,
        0.3368614597963451,
        0.5702222908466830,
        0.0146469500556540,
        0.2983728821362581,
        0.6869801678080881,
        0.0012683309328720,
        0.1189744976969570,
        0.8797571713701710,
        0.4889639103621790,
        0.0220721792756430,
        0.4889639103621780,
        0.4889639103621790,
        0.4889639103621790,
        0.0220721792756420,
        0.4176447193404539,
        0.1647105613190920,
        0.4176447193404539,
        0.4176447193404539,
        0.4176447193404539,
        0.1647105613190920,
        0.2734775283088389,
        0.4530449433823229,
        0.2734775283088380,
        0.2734775283088389,
        0.2734775283088389,
        0.4530449433823220,
        0.1772055324125430,
        0.6455889351749130,
        0.1772055324125440,
        0.1772055324125430,
        0.1772055324125430,
        0.6455889351749140,
        0.0617998830908730,
        0.8764002338182550,
        0.0617998830908719,
        0.0617998830908730,
        0.0617998830908730,
        0.8764002338182539,
        0.0193909612487010,
        0.9612180775025981,
        0.0193909612487010,
        0.0193909612487010,
        0.0193909612487010,
        0.9612180775025981,
        0.0571247574036480,
        0.7706085547749961,
        0.1722666878213560,
        0.1722666878213560,
        0.7706085547749961,
        0.0571247574036480,
        0.7706085547749961,
        0.0571247574036480,
        0.1722666878213560,
        0.1722666878213560,
        0.0571247574036480,
        0.7706085547749961,
        0.7706085547749961,
        0.1722666878213560,
        0.0571247574036480,
        0.0929162493569720,
        0.5702222908466830,
        0.3368614597963451,
        0.3368614597963451,
        0.5702222908466830,
        0.0929162493569720,
        0.5702222908466830,
        0.0929162493569720,
        0.3368614597963451,
        0.3368614597963451,
        0.0929162493569720,
        0.5702222908466830,
        0.5702222908466830,
        0.3368614597963451,
        0.0929162493569720,
        0.0146469500556540,
        0.6869801678080881,
        0.2983728821362581,
        0.2983728821362581,
        0.6869801678080881,
        0.0146469500556540,
        0.6869801678080881,
        0.0146469500556540,
        0.2983728821362581,
        0.2983728821362581,
        0.0146469500556540,
        0.6869801678080881,
        0.6869801678080881,
        0.2983728821362581,
        0.0146469500556540,
        0.0012683309328720,
        0.8797571713701710,
        0.1189744976969570,
        0.1189744976969570,
        0.8797571713701710,
        0.0012683309328720,
        0.8797571713701710,
        0.0012683309328720,
        0.1189744976969570,
        0.1189744976969570,
        0.0012683309328720,
        0.8797571713701710,
        0.8797571713701710,
        0.1189744976969570,
        0.0012683309328720,
        # Coordinates for number 48
        -0.0139458337164860,
        0.5069729168582429,
        0.5069729168582430,
        0.1371872914339550,
        0.4314063542830230,
        0.4314063542830220,
        0.4446127103057109,
        0.2776936448471439,
        0.2776936448471450,
        0.7470702179174919,
        0.1264648910412540,
        0.1264648910412541,
        0.8583832280506280,
        0.0708083859746860,
        0.0708083859746859,
        0.9620696595178530,
        0.0189651702410730,
        0.0189651702410741,
        0.1337341619666210,
        0.2613113711400871,
        0.6049544668932921,
        0.0363666773969170,
        0.3880467670902690,
        0.5755865555128141,
        -0.0101748831265710,
        0.2857122200499159,
        0.7244626630766551,
        0.0368438698758780,
        0.2155996640722840,
        0.7475564660518381,
        0.0124598093311990,
        0.1035756165763860,
        0.8839645740924151,
        0.5069729168582429,
        -0.0139458337164860,
        0.5069729168582430,
        0.5069729168582429,
        0.5069729168582429,
        -0.0139458337164859,
        0.4314063542830230,
        0.1371872914339550,
        0.4314063542830220,
        0.4314063542830230,
        0.4314063542830230,
        0.1371872914339540,
        0.2776936448471439,
        0.4446127103057109,
        0.2776936448471450,
        0.2776936448471439,
        0.2776936448471439,
        0.4446127103057120,
        0.1264648910412540,
        0.7470702179174919,
        0.1264648910412541,
        0.1264648910412540,
        0.1264648910412540,
        0.7470702179174919,
        0.0708083859746860,
        0.8583832280506280,
        0.0708083859746859,
        0.0708083859746860,
        0.0708083859746860,
        0.8583832280506280,
        0.0189651702410730,
        0.9620696595178530,
        0.0189651702410741,
        0.0189651702410730,
        0.0189651702410730,
        0.9620696595178540,
        0.1337341619666210,
        0.6049544668932911,
        0.2613113711400880,
        0.2613113711400871,
        0.6049544668932911,
        0.1337341619666220,
        0.6049544668932911,
        0.1337341619666210,
        0.2613113711400880,
        0.2613113711400871,
        0.1337341619666210,
        0.6049544668932921,
        0.6049544668932911,
        0.2613113711400871,
        0.1337341619666220,
        0.0363666773969170,
        0.5755865555128141,
        0.3880467670902690,
        0.3880467670902690,
        0.5755865555128141,
        0.0363666773969170,
        0.5755865555128141,
        0.0363666773969170,
        0.3880467670902690,
        0.3880467670902690,
        0.0363666773969170,
        0.5755865555128141,
        0.5755865555128141,
        0.3880467670902690,
        0.0363666773969170,
        -0.0101748831265710,
        0.7244626630766550,
        0.2857122200499160,
        0.2857122200499159,
        0.7244626630766550,
        -0.0101748831265709,
        0.7244626630766550,
        -0.0101748831265710,
        0.2857122200499160,
        0.2857122200499159,
        -0.0101748831265710,
        0.7244626630766551,
        0.7244626630766550,
        0.2857122200499159,
        -0.0101748831265709,
        0.0368438698758780,
        0.7475564660518381,
        0.2155996640722840,
        0.2155996640722840,
        0.7475564660518381,
        0.0368438698758780,
        0.7475564660518381,
        0.0368438698758780,
        0.2155996640722840,
        0.2155996640722840,
        0.0368438698758780,
        0.7475564660518381,
        0.7475564660518381,
        0.2155996640722840,
        0.0368438698758780,
        0.0124598093311990,
        0.8839645740924161,
        0.1035756165763850,
        0.1035756165763860,
        0.8839645740924161,
        0.0124598093311980,
        0.8839645740924161,
        0.0124598093311990,
        0.1035756165763850,
        0.1035756165763860,
        0.0124598093311990,
        0.8839645740924151,
        0.8839645740924161,
        0.1035756165763860,
        0.0124598093311980,
        # Coordinates for number 52
        0.3333333333333329,
        0.3333333333333329,
        0.3333333333333340,
        0.0052389161031230,
        0.4973805419484380,
        0.4973805419484390,
        0.1730611229012950,
        0.4134694385493519,
        0.4134694385493530,
        0.0590828018660170,
        0.4704585990669911,
        0.4704585990669921,
        0.5188925000609579,
        0.2405537499695210,
        0.2405537499695210,
        0.7040684115548540,
        0.1479657942225730,
        0.1479657942225730,
        0.8490696246850519,
        0.0754651876574740,
        0.0754651876574740,
        0.9668071947539501,
        0.0165964026230250,
        0.0165964026230250,
        0.1035756922452520,
        0.2965555965798871,
        0.5998687111748609,
        0.0200834116554160,
        0.3377230634030791,
        0.6421935249415049,
        -0.0043410026141390,
        0.2047482816428120,
        0.7995927209713269,
        0.0419417864680100,
        0.1893584921306230,
        0.7686997214013669,
        0.0143173202306810,
        0.0852836156826570,
        0.9003990640866621,
        0.4973805419484380,
        0.0052389161031230,
        0.4973805419484390,
        0.4973805419484380,
        0.4973805419484380,
        0.0052389161031240,
        0.4134694385493519,
        0.1730611229012950,
        0.4134694385493530,
        0.4134694385493519,
        0.4134694385493519,
        0.1730611229012960,
        0.4704585990669911,
        0.0590828018660170,
        0.4704585990669921,
        0.4704585990669911,
        0.4704585990669911,
        0.0590828018660180,
        0.2405537499695210,
        0.5188925000609579,
        0.2405537499695210,
        0.2405537499695210,
        0.2405537499695210,
        0.5188925000609579,
        0.1479657942225730,
        0.7040684115548540,
        0.1479657942225730,
        0.1479657942225730,
        0.1479657942225730,
        0.7040684115548540,
        0.0754651876574740,
        0.8490696246850519,
        0.0754651876574740,
        0.0754651876574740,
        0.0754651876574740,
        0.8490696246850519,
        0.0165964026230250,
        0.9668071947539501,
        0.0165964026230250,
        0.0165964026230250,
        0.0165964026230250,
        0.9668071947539501,
        0.1035756922452520,
        0.5998687111748609,
        0.2965555965798871,
        0.2965555965798871,
        0.5998687111748609,
        0.1035756922452520,
        0.5998687111748609,
        0.1035756922452520,
        0.2965555965798871,
        0.2965555965798871,
        0.1035756922452520,
        0.5998687111748609,
        0.5998687111748609,
        0.2965555965798871,
        0.1035756922452520,
        0.0200834116554160,
        0.6421935249415049,
        0.3377230634030791,
        0.3377230634030791,
        0.6421935249415049,
        0.0200834116554161,
        0.6421935249415049,
        0.0200834116554160,
        0.3377230634030791,
        0.3377230634030791,
        0.0200834116554160,
        0.6421935249415049,
        0.6421935249415049,
        0.3377230634030791,
        0.0200834116554161,
        -0.0043410026141390,
        0.7995927209713270,
        0.2047482816428119,
        0.2047482816428120,
        0.7995927209713270,
        -0.0043410026141391,
        0.7995927209713270,
        -0.0043410026141390,
        0.2047482816428119,
        0.2047482816428120,
        -0.0043410026141390,
        0.7995927209713269,
        0.7995927209713270,
        0.2047482816428120,
        -0.0043410026141391,
        0.0419417864680100,
        0.7686997214013679,
        0.1893584921306220,
        0.1893584921306230,
        0.7686997214013679,
        0.0419417864680090,
        0.7686997214013679,
        0.0419417864680100,
        0.1893584921306220,
        0.1893584921306230,
        0.0419417864680100,
        0.7686997214013669,
        0.7686997214013679,
        0.1893584921306230,
        0.0419417864680090,
        0.0143173202306810,
        0.9003990640866611,
        0.0852836156826580,
        0.0852836156826570,
        0.9003990640866611,
        0.0143173202306820,
        0.9003990640866611,
        0.0143173202306810,
        0.0852836156826580,
        0.0852836156826570,
        0.0143173202306810,
        0.9003990640866621,
        0.9003990640866611,
        0.0852836156826570,
        0.0143173202306820,
        # Coordinates for number 61
        0.3333333333333329,
        0.3333333333333329,
        0.3333333333333340,
        0.0056589188864520,
        0.4971705405567740,
        0.4971705405567740,
        0.0356473547507510,
        0.4821763226246250,
        0.4821763226246240,
        0.0995200619584370,
        0.4502399690207821,
        0.4502399690207811,
        0.1994675212452060,
        0.4002662393773970,
        0.4002662393773969,
        0.4957174640580951,
        0.2521412679709530,
        0.2521412679709520,
        0.6759059906830770,
        0.1620470046584610,
        0.1620470046584620,
        0.8482482354785080,
        0.0758758822607460,
        0.0758758822607460,
        0.9686905460643560,
        0.0156547269678220,
        0.0156547269678220,
        0.0101869288269190,
        0.3343198673636580,
        0.6554932038094229,
        0.1354408716710360,
        0.2922215377969439,
        0.5723375905320201,
        0.0544239242905830,
        0.3195748854231901,
        0.6260011902862271,
        0.0128685608336370,
        0.1907042241922920,
        0.7964272149740709,
        0.0671657824135240,
        0.1804832116487460,
        0.7523510059377299,
        0.0146631822248280,
        0.0807113136795640,
        0.9046255040956080,
        0.4971705405567740,
        0.0056589188864520,
        0.4971705405567740,
        0.4971705405567740,
        0.4971705405567740,
        0.0056589188864520,
        0.4821763226246250,
        0.0356473547507510,
        0.4821763226246240,
        0.4821763226246250,
        0.4821763226246250,
        0.0356473547507500,
        0.4502399690207821,
        0.0995200619584370,
        0.4502399690207811,
        0.4502399690207821,
        0.4502399690207821,
        0.0995200619584360,
        0.4002662393773970,
        0.1994675212452060,
        0.4002662393773969,
        0.4002662393773970,
        0.4002662393773970,
        0.1994675212452059,
        0.2521412679709530,
        0.4957174640580951,
        0.2521412679709520,
        0.2521412679709530,
        0.2521412679709530,
        0.4957174640580941,
        0.1620470046584610,
        0.6759059906830770,
        0.1620470046584620,
        0.1620470046584610,
        0.1620470046584610,
        0.6759059906830780,
        0.0758758822607460,
        0.8482482354785080,
        0.0758758822607460,
        0.0758758822607460,
        0.0758758822607460,
        0.8482482354785080,
        0.0156547269678220,
        0.9686905460643560,
        0.0156547269678220,
        0.0156547269678220,
        0.0156547269678220,
        0.9686905460643560,
        0.0101869288269190,
        0.6554932038094229,
        0.3343198673636580,
        0.3343198673636580,
        0.6554932038094229,
        0.0101869288269190,
        0.6554932038094229,
        0.0101869288269190,
        0.3343198673636580,
        0.3343198673636580,
        0.0101869288269190,
        0.6554932038094229,
        0.6554932038094229,
        0.3343198673636580,
        0.0101869288269190,
        0.1354408716710360,
        0.5723375905320201,
        0.2922215377969439,
        0.2922215377969439,
        0.5723375905320201,
        0.1354408716710360,
        0.5723375905320201,
        0.1354408716710360,
        0.2922215377969439,
        0.2922215377969439,
        0.1354408716710360,
        0.5723375905320201,
        0.5723375905320201,
        0.2922215377969439,
        0.1354408716710360,
        0.0544239242905830,
        0.6260011902862281,
        0.3195748854231890,
        0.3195748854231901,
        0.6260011902862281,
        0.0544239242905820,
        0.6260011902862281,
        0.0544239242905830,
        0.3195748854231890,
        0.3195748854231901,
        0.0544239242905830,
        0.6260011902862271,
        0.6260011902862281,
        0.3195748854231901,
        0.0544239242905820,
        0.0128685608336370,
        0.7964272149740709,
        0.1907042241922920,
        0.1907042241922920,
        0.7964272149740709,
        0.0128685608336370,
        0.7964272149740709,
        0.0128685608336370,
        0.1907042241922920,
        0.1907042241922920,
        0.0128685608336370,
        0.7964272149740709,
        0.7964272149740709,
        0.1907042241922920,
        0.0128685608336370,
        0.0671657824135240,
        0.7523510059377289,
        0.1804832116487470,
        0.1804832116487460,
        0.7523510059377289,
        0.0671657824135250,
        0.7523510059377289,
        0.0671657824135240,
        0.1804832116487470,
        0.1804832116487460,
        0.0671657824135240,
        0.7523510059377299,
        0.7523510059377289,
        0.1804832116487460,
        0.0671657824135250,
        0.0146631822248280,
        0.9046255040956080,
        0.0807113136795640,
        0.0807113136795640,
        0.9046255040956080,
        0.0146631822248280,
        0.9046255040956080,
        0.0146631822248280,
        0.0807113136795640,
        0.0807113136795640,
        0.0146631822248280,
        0.9046255040956080,
        0.9046255040956080,
        0.0807113136795640,
        0.0146631822248280,
        # Coordinates for number 70
        0.3333333333333329,
        0.3333333333333329,
        0.3333333333333340,
        0.0133103827381570,
        0.4933448086309210,
        0.4933448086309220,
        0.0615788115160860,
        0.4692105942419570,
        0.4692105942419570,
        0.1274372082259890,
        0.4362813958870060,
        0.4362813958870050,
        0.2103076586531680,
        0.3948461706734160,
        0.3948461706734160,
        0.5004108623936861,
        0.2497945688031570,
        0.2497945688031570,
        0.6771356125123150,
        0.1614321937438430,
        0.1614321937438420,
        0.8468035450292570,
        0.0765982274853710,
        0.0765982274853720,
        0.9514951212930999,
        0.0242524393534500,
        0.0242524393534500,
        0.9137072655660710,
        0.0431463672169650,
        0.0431463672169640,
        0.0084305362024200,
        0.3589114949409440,
        0.6326579688566361,
        0.1311865517371880,
        0.2944024767519571,
        0.5744109715108550,
        0.0502031515656750,
        0.3250178016418139,
        0.6247790467925110,
        0.0663292638109160,
        0.1847375596660460,
        0.7489331765230381,
        0.0119961945662360,
        0.2187968000133210,
        0.7692070054204429,
        0.0148581005901250,
        0.1011795971364080,
        0.8839623022734669,
        -0.0352220152879490,
        0.0208747552825860,
        1.0143472600053629,
        0.4933448086309210,
        0.0133103827381570,
        0.4933448086309220,
        0.4933448086309210,
        0.4933448086309210,
        0.0133103827381580,
        0.4692105942419570,
        0.0615788115160860,
        0.4692105942419570,
        0.4692105942419570,
        0.4692105942419570,
        0.0615788115160860,
        0.4362813958870060,
        0.1274372082259890,
        0.4362813958870050,
        0.4362813958870060,
        0.4362813958870060,
        0.1274372082259880,
        0.3948461706734160,
        0.2103076586531680,
        0.3948461706734160,
        0.3948461706734160,
        0.3948461706734160,
        0.2103076586531680,
        0.2497945688031570,
        0.5004108623936861,
        0.2497945688031570,
        0.2497945688031570,
        0.2497945688031570,
        0.5004108623936861,
        0.1614321937438430,
        0.6771356125123150,
        0.1614321937438420,
        0.1614321937438430,
        0.1614321937438430,
        0.6771356125123140,
        0.0765982274853710,
        0.8468035450292570,
        0.0765982274853720,
        0.0765982274853710,
        0.0765982274853710,
        0.8468035450292580,
        0.0242524393534500,
        0.9514951212930999,
        0.0242524393534500,
        0.0242524393534500,
        0.0242524393534500,
        0.9514951212930999,
        0.0431463672169650,
        0.9137072655660710,
        0.0431463672169640,
        0.0431463672169650,
        0.0431463672169650,
        0.9137072655660700,
        0.0084305362024200,
        0.6326579688566361,
        0.3589114949409440,
        0.3589114949409440,
        0.6326579688566361,
        0.0084305362024200,
        0.6326579688566361,
        0.0084305362024200,
        0.3589114949409440,
        0.3589114949409440,
        0.0084305362024200,
        0.6326579688566361,
        0.6326579688566361,
        0.3589114949409440,
        0.0084305362024200,
        0.1311865517371880,
        0.5744109715108550,
        0.2944024767519571,
        0.2944024767519571,
        0.5744109715108550,
        0.1311865517371880,
        0.5744109715108550,
        0.1311865517371880,
        0.2944024767519571,
        0.2944024767519571,
        0.1311865517371880,
        0.5744109715108550,
        0.5744109715108550,
        0.2944024767519571,
        0.1311865517371880,
        0.0502031515656750,
        0.6247790467925119,
        0.3250178016418130,
        0.3250178016418139,
        0.6247790467925119,
        0.0502031515656741,
        0.6247790467925119,
        0.0502031515656750,
        0.3250178016418130,
        0.3250178016418139,
        0.0502031515656750,
        0.6247790467925110,
        0.6247790467925119,
        0.3250178016418139,
        0.0502031515656741,
        0.0663292638109160,
        0.7489331765230371,
        0.1847375596660470,
        0.1847375596660460,
        0.7489331765230371,
        0.0663292638109170,
        0.7489331765230371,
        0.0663292638109160,
        0.1847375596660470,
        0.1847375596660460,
        0.0663292638109160,
        0.7489331765230381,
        0.7489331765230371,
        0.1847375596660460,
        0.0663292638109170,
        0.0119961945662360,
        0.7692070054204429,
        0.2187968000133211,
        0.2187968000133210,
        0.7692070054204429,
        0.0119961945662360,
        0.7692070054204429,
        0.0119961945662360,
        0.2187968000133211,
        0.2187968000133210,
        0.0119961945662360,
        0.7692070054204429,
        0.7692070054204429,
        0.2187968000133210,
        0.0119961945662360,
        0.0148581005901250,
        0.8839623022734669,
        0.1011795971364080,
        0.1011795971364080,
        0.8839623022734669,
        0.0148581005901250,
        0.8839623022734669,
        0.0148581005901250,
        0.1011795971364080,
        0.1011795971364080,
        0.0148581005901250,
        0.8839623022734669,
        0.8839623022734669,
        0.1011795971364080,
        0.0148581005901250,
        -0.0352220152879490,
        1.0143472600053629,
        0.0208747552825861,
        0.0208747552825860,
        1.0143472600053629,
        -0.0352220152879489,
        1.0143472600053629,
        -0.0352220152879490,
        0.0208747552825861,
        0.0208747552825860,
        -0.0352220152879490,
        1.0143472600053629,
        1.0143472600053629,
        0.0208747552825860,
        -0.0352220152879489,
        # Coordinates for number 73
        0.3333333333333329,
        0.3333333333333329,
        0.3333333333333340,
        0.0207800258539870,
        0.4896099870730061,
        0.4896099870730070,
        0.0909262146042150,
        0.4545368926978930,
        0.4545368926978920,
        0.1971666387011380,
        0.4014166806494310,
        0.4014166806494311,
        0.4888966911938051,
        0.2555516544030981,
        0.2555516544030970,
        0.6458441156957409,
        0.1770779421521300,
        0.1770779421521290,
        0.7798778935440960,
        0.1100610532279520,
        0.1100610532279520,
        0.8889427514963210,
        0.0555286242518400,
        0.0555286242518390,
        0.9747562724455430,
        0.0126218637772290,
        0.0126218637772280,
        0.0036114178484120,
        0.3957547873569431,
        0.6006337947946450,
        0.1344667545307800,
        0.3079299838804361,
        0.5576032615887841,
        0.0144460257761150,
        0.2645669484065200,
        0.7209870258173649,
        0.0469335788381780,
        0.3585393522059511,
        0.5945270689558711,
        0.0028611203505670,
        0.1578074059685950,
        0.8393314736808381,
        0.2238614240979160,
        0.0750505969759110,
        0.7010879789261731,
        0.0346470748167600,
        0.1424216011133830,
        0.8229313240698570,
        0.0101611192962780,
        0.0654946280829380,
        0.9243442526207841,
        0.4896099870730061,
        0.0207800258539870,
        0.4896099870730070,
        0.4896099870730061,
        0.4896099870730061,
        0.0207800258539880,
        0.4545368926978930,
        0.0909262146042150,
        0.4545368926978920,
        0.4545368926978930,
        0.4545368926978930,
        0.0909262146042140,
        0.4014166806494310,
        0.1971666387011380,
        0.4014166806494311,
        0.4014166806494310,
        0.4014166806494310,
        0.1971666387011380,
        0.2555516544030981,
        0.4888966911938051,
        0.2555516544030970,
        0.2555516544030981,
        0.2555516544030981,
        0.4888966911938040,
        0.1770779421521300,
        0.6458441156957409,
        0.1770779421521290,
        0.1770779421521300,
        0.1770779421521300,
        0.6458441156957399,
        0.1100610532279520,
        0.7798778935440960,
        0.1100610532279520,
        0.1100610532279520,
        0.1100610532279520,
        0.7798778935440960,
        0.0555286242518400,
        0.8889427514963210,
        0.0555286242518390,
        0.0555286242518400,
        0.0555286242518400,
        0.8889427514963200,
        0.0126218637772290,
        0.9747562724455430,
        0.0126218637772280,
        0.0126218637772290,
        0.0126218637772290,
        0.9747562724455420,
        0.0036114178484120,
        0.6006337947946450,
        0.3957547873569431,
        0.3957547873569431,
        0.6006337947946450,
        0.0036114178484120,
        0.6006337947946450,
        0.0036114178484120,
        0.3957547873569431,
        0.3957547873569431,
        0.0036114178484120,
        0.6006337947946450,
        0.6006337947946450,
        0.3957547873569431,
        0.0036114178484120,
        0.1344667545307800,
        0.5576032615887841,
        0.3079299838804360,
        0.3079299838804361,
        0.5576032615887841,
        0.1344667545307799,
        0.5576032615887841,
        0.1344667545307800,
        0.3079299838804360,
        0.3079299838804361,
        0.1344667545307800,
        0.5576032615887841,
        0.5576032615887841,
        0.3079299838804361,
        0.1344667545307799,
        0.0144460257761150,
        0.7209870258173649,
        0.2645669484065200,
        0.2645669484065200,
        0.7209870258173649,
        0.0144460257761150,
        0.7209870258173649,
        0.0144460257761150,
        0.2645669484065200,
        0.2645669484065200,
        0.0144460257761150,
        0.7209870258173649,
        0.7209870258173649,
        0.2645669484065200,
        0.0144460257761150,
        0.0469335788381780,
        0.5945270689558711,
        0.3585393522059510,
        0.3585393522059511,
        0.5945270689558711,
        0.0469335788381779,
        0.5945270689558711,
        0.0469335788381780,
        0.3585393522059510,
        0.3585393522059511,
        0.0469335788381780,
        0.5945270689558711,
        0.5945270689558711,
        0.3585393522059511,
        0.0469335788381779,
        0.0028611203505670,
        0.8393314736808391,
        0.1578074059685940,
        0.1578074059685950,
        0.8393314736808391,
        0.0028611203505660,
        0.8393314736808391,
        0.0028611203505670,
        0.1578074059685940,
        0.1578074059685950,
        0.0028611203505670,
        0.8393314736808381,
        0.8393314736808391,
        0.1578074059685950,
        0.0028611203505660,
        0.2238614240979160,
        0.7010879789261731,
        0.0750505969759110,
        0.0750505969759110,
        0.7010879789261731,
        0.2238614240979160,
        0.7010879789261731,
        0.2238614240979160,
        0.0750505969759110,
        0.0750505969759110,
        0.2238614240979160,
        0.7010879789261731,
        0.7010879789261731,
        0.0750505969759110,
        0.2238614240979160,
        0.0346470748167600,
        0.8229313240698570,
        0.1424216011133830,
        0.1424216011133830,
        0.8229313240698570,
        0.0346470748167600,
        0.8229313240698570,
        0.0346470748167600,
        0.1424216011133830,
        0.1424216011133830,
        0.0346470748167600,
        0.8229313240698570,
        0.8229313240698570,
        0.1424216011133830,
        0.0346470748167600,
        0.0101611192962780,
        0.9243442526207841,
        0.0654946280829380,
        0.0654946280829380,
        0.9243442526207841,
        0.0101611192962780,
        0.9243442526207841,
        0.0101611192962780,
        0.0654946280829380,
        0.0654946280829380,
        0.0101611192962780,
        0.9243442526207841,
        0.9243442526207841,
        0.0654946280829380,
        0.0101611192962780,
        # Coordinates for number 79
        0.3333333333333329,
        0.3333333333333329,
        0.3333333333333340,
        -0.0019009287044000,
        0.5009504643522001,
        0.5009504643522000,
        0.0235740841305430,
        0.4882129579347291,
        0.4882129579347280,
        0.0897266360994350,
        0.4551366819502831,
        0.4551366819502821,
        0.1960074813634210,
        0.4019962593182890,
        0.4019962593182901,
        0.4882141804811571,
        0.2558929097594209,
        0.2558929097594220,
        0.6470234880097881,
        0.1764882559951060,
        0.1764882559951060,
        0.7916582893264830,
        0.1041708553367580,
        0.1041708553367590,
        0.8938620723181400,
        0.0530689638409300,
        0.0530689638409300,
        0.9167625696079421,
        0.0416187151960290,
        0.0416187151960290,
        0.9768361571863560,
        0.0115819214068220,
        0.0115819214068220,
        0.0487415836648390,
        0.3448557702290010,
        0.6064026461061600,
        0.0063141159486050,
        0.3778432695948540,
        0.6158426144565409,
        0.1343165205473480,
        0.3066354790623570,
        0.5590480003902950,
        0.0139738939623920,
        0.2494193627747420,
        0.7366067432628660,
        0.0755491329097640,
        0.2127757248028020,
        0.7116751422874340,
        -0.0083681532082270,
        0.1469654360532390,
        0.8614027171549879,
        0.0266860632587140,
        0.1377269788289230,
        0.8355869579123631,
        0.0105477192941410,
        0.0596961091490070,
        0.9297561715568520,
        0.5009504643522001,
        -0.0019009287044000,
        0.5009504643522000,
        0.5009504643522001,
        0.5009504643522001,
        -0.0019009287044001,
        0.4882129579347291,
        0.0235740841305430,
        0.4882129579347280,
        0.4882129579347291,
        0.4882129579347291,
        0.0235740841305420,
        0.4551366819502831,
        0.0897266360994350,
        0.4551366819502821,
        0.4551366819502831,
        0.4551366819502831,
        0.0897266360994340,
        0.4019962593182890,
        0.1960074813634210,
        0.4019962593182901,
        0.4019962593182890,
        0.4019962593182890,
        0.1960074813634221,
        0.2558929097594209,
        0.4882141804811571,
        0.2558929097594220,
        0.2558929097594209,
        0.2558929097594209,
        0.4882141804811581,
        0.1764882559951060,
        0.6470234880097881,
        0.1764882559951060,
        0.1764882559951060,
        0.1764882559951060,
        0.6470234880097881,
        0.1041708553367580,
        0.7916582893264830,
        0.1041708553367590,
        0.1041708553367580,
        0.1041708553367580,
        0.7916582893264840,
        0.0530689638409300,
        0.8938620723181400,
        0.0530689638409300,
        0.0530689638409300,
        0.0530689638409300,
        0.8938620723181400,
        0.0416187151960290,
        0.9167625696079421,
        0.0416187151960290,
        0.0416187151960290,
        0.0416187151960290,
        0.9167625696079421,
        0.0115819214068220,
        0.9768361571863560,
        0.0115819214068220,
        0.0115819214068220,
        0.0115819214068220,
        0.9768361571863560,
        0.0487415836648390,
        0.6064026461061600,
        0.3448557702290010,
        0.3448557702290010,
        0.6064026461061600,
        0.0487415836648390,
        0.6064026461061600,
        0.0487415836648390,
        0.3448557702290010,
        0.3448557702290010,
        0.0487415836648390,
        0.6064026461061600,
        0.6064026461061600,
        0.3448557702290010,
        0.0487415836648390,
        0.0063141159486050,
        0.6158426144565409,
        0.3778432695948541,
        0.3778432695948540,
        0.6158426144565409,
        0.0063141159486050,
        0.6158426144565409,
        0.0063141159486050,
        0.3778432695948541,
        0.3778432695948540,
        0.0063141159486050,
        0.6158426144565409,
        0.6158426144565409,
        0.3778432695948540,
        0.0063141159486050,
        0.1343165205473480,
        0.5590480003902950,
        0.3066354790623570,
        0.3066354790623570,
        0.5590480003902950,
        0.1343165205473480,
        0.5590480003902950,
        0.1343165205473480,
        0.3066354790623570,
        0.3066354790623570,
        0.1343165205473480,
        0.5590480003902950,
        0.5590480003902950,
        0.3066354790623570,
        0.1343165205473480,
        0.0139738939623920,
        0.7366067432628660,
        0.2494193627747420,
        0.2494193627747420,
        0.7366067432628660,
        0.0139738939623920,
        0.7366067432628660,
        0.0139738939623920,
        0.2494193627747420,
        0.2494193627747420,
        0.0139738939623920,
        0.7366067432628660,
        0.7366067432628660,
        0.2494193627747420,
        0.0139738939623920,
        0.0755491329097640,
        0.7116751422874340,
        0.2127757248028020,
        0.2127757248028020,
        0.7116751422874340,
        0.0755491329097640,
        0.7116751422874340,
        0.0755491329097640,
        0.2127757248028020,
        0.2127757248028020,
        0.0755491329097640,
        0.7116751422874340,
        0.7116751422874340,
        0.2127757248028020,
        0.0755491329097640,
        -0.0083681532082270,
        0.8614027171549869,
        0.1469654360532401,
        0.1469654360532390,
        0.8614027171549869,
        -0.0083681532082260,
        0.8614027171549869,
        -0.0083681532082270,
        0.1469654360532401,
        0.1469654360532390,
        -0.0083681532082270,
        0.8614027171549879,
        0.8614027171549869,
        0.1469654360532390,
        -0.0083681532082260,
        0.0266860632587140,
        0.8355869579123631,
        0.1377269788289230,
        0.1377269788289230,
        0.8355869579123631,
        0.0266860632587139,
        0.8355869579123631,
        0.0266860632587140,
        0.1377269788289230,
        0.1377269788289230,
        0.0266860632587140,
        0.8355869579123631,
        0.8355869579123631,
        0.1377269788289230,
        0.0266860632587139,
        0.0105477192941410,
        0.9297561715568530,
        0.0596961091490060,
        0.0596961091490070,
        0.9297561715568530,
        0.0105477192941400,
        0.9297561715568530,
        0.0105477192941410,
        0.0596961091490060,
        0.0596961091490070,
        0.0105477192941410,
        0.9297561715568520,
        0.9297561715568530,
        0.0596961091490070,
        0.0105477192941400,
    ]
)

#
#  Gauss weights for symmetric quadrature rules on a triangle
#  Note: Weights sum up to 1, not 1/2!
#
weights = _np.array(
    [
        # Weights for number 1
        1.0000000000000000,
        # Weights for number 3
        0.3333333333333329,
        0.3333333333333329,
        0.3333333333333329,
        # Weights for number 4
        -0.5625000000000000,
        0.5208333333333330,
        0.5208333333333330,
        0.5208333333333330,
        # Weights for number 6
        0.2233815896780110,
        0.1099517436553220,
        0.2233815896780110,
        0.2233815896780110,
        0.1099517436553220,
        0.1099517436553220,
        # Weights for number 7
        0.2250000000000000,
        0.1323941527885060,
        0.1259391805448270,
        0.1323941527885060,
        0.1323941527885060,
        0.1259391805448270,
        0.1259391805448270,
        # Weights for number 9
        0.2059505047608870,
        0.2059505047608870,
        0.2059505047608870,
        0.0636914142862230,
        0.0636914142862230,
        0.0636914142862230,
        0.0636914142862230,
        0.0636914142862230,
        0.0636914142862230,
        # Weights for number 12
        0.1167862757263790,
        0.0508449063702070,
        0.0828510756183740,
        0.1167862757263790,
        0.1167862757263790,
        0.0508449063702070,
        0.0508449063702070,
        0.0828510756183740,
        0.0828510756183740,
        0.0828510756183740,
        0.0828510756183740,
        0.0828510756183740,
        # Weights for number 13
        -0.1495700444676820,
        0.1756152574332080,
        0.0533472356088380,
        0.0771137608902570,
        0.1756152574332080,
        0.1756152574332080,
        0.0533472356088380,
        0.0533472356088380,
        0.0771137608902570,
        0.0771137608902570,
        0.0771137608902570,
        0.0771137608902570,
        0.0771137608902570,
        # Weights for number 15
        0.0530778017902330,
        0.0530778017902330,
        0.0530778017902330,
        0.0708530836921360,
        0.0708530836921360,
        0.0708530836921360,
        0.0708530836921360,
        0.0708530836921360,
        0.0708530836921360,
        0.0692746820794150,
        0.0692746820794150,
        0.0692746820794150,
        0.0692746820794150,
        0.0692746820794150,
        0.0692746820794150,
        # Weights for number 16
        0.1443156076777870,
        0.0950916342672850,
        0.1032173705347180,
        0.0324584976231980,
        0.0272303141744350,
        0.0950916342672850,
        0.0950916342672850,
        0.1032173705347180,
        0.1032173705347180,
        0.0324584976231980,
        0.0324584976231980,
        0.0272303141744350,
        0.0272303141744350,
        0.0272303141744350,
        0.0272303141744350,
        0.0272303141744350,
        # Weights for number 19
        0.0971357962827990,
        0.0313347002271390,
        0.0778275410047740,
        0.0796477389272100,
        0.0255776756586980,
        0.0432835393772890,
        0.0313347002271390,
        0.0313347002271390,
        0.0778275410047740,
        0.0778275410047740,
        0.0796477389272100,
        0.0796477389272100,
        0.0255776756586980,
        0.0255776756586980,
        0.0432835393772890,
        0.0432835393772890,
        0.0432835393772890,
        0.0432835393772890,
        0.0432835393772890,
        # Weights for number 21
        0.0516172025690210,
        0.0516172025690210,
        0.0516172025690210,
        0.0940800734583560,
        0.0940800734583560,
        0.0940800734583560,
        0.0259935710323200,
        0.0259935710323200,
        0.0259935710323200,
        0.0454695380476190,
        0.0454695380476190,
        0.0454695380476190,
        0.0454695380476190,
        0.0454695380476190,
        0.0454695380476190,
        0.0353517050891990,
        0.0353517050891990,
        0.0353517050891990,
        0.0353517050891990,
        0.0353517050891990,
        0.0353517050891990,
        # Weights for number 25
        0.0908179903827540,
        0.0367259577564670,
        0.0453210594355280,
        0.0727579168454200,
        0.0283272425310570,
        0.0094216669637330,
        0.0367259577564670,
        0.0367259577564670,
        0.0453210594355280,
        0.0453210594355280,
        0.0727579168454200,
        0.0727579168454200,
        0.0727579168454200,
        0.0727579168454200,
        0.0727579168454200,
        0.0283272425310570,
        0.0283272425310570,
        0.0283272425310570,
        0.0283272425310570,
        0.0283272425310570,
        0.0094216669637330,
        0.0094216669637330,
        0.0094216669637330,
        0.0094216669637330,
        0.0094216669637330,
        # Weights for number 27
        0.0009270063289610,
        0.0771495349148130,
        0.0593229773807740,
        0.0361845405034180,
        0.0136597310026780,
        0.0523371119622040,
        0.0207076596391410,
        0.0009270063289610,
        0.0009270063289610,
        0.0771495349148130,
        0.0771495349148130,
        0.0593229773807740,
        0.0593229773807740,
        0.0361845405034180,
        0.0361845405034180,
        0.0136597310026780,
        0.0136597310026780,
        0.0523371119622040,
        0.0523371119622040,
        0.0523371119622040,
        0.0523371119622040,
        0.0523371119622040,
        0.0207076596391410,
        0.0207076596391410,
        0.0207076596391410,
        0.0207076596391410,
        0.0207076596391410,
        # Weights for number 33
        0.0257310664404550,
        0.0436925445380380,
        0.0628582242178850,
        0.0347961129307090,
        0.0061662610515590,
        0.0403715577663810,
        0.0223567732023030,
        0.0173162311086590,
        0.0257310664404550,
        0.0257310664404550,
        0.0436925445380380,
        0.0436925445380380,
        0.0628582242178850,
        0.0628582242178850,
        0.0347961129307090,
        0.0347961129307090,
        0.0061662610515590,
        0.0061662610515590,
        0.0403715577663810,
        0.0403715577663810,
        0.0403715577663810,
        0.0403715577663810,
        0.0403715577663810,
        0.0223567732023030,
        0.0223567732023030,
        0.0223567732023030,
        0.0223567732023030,
        0.0223567732023030,
        0.0173162311086590,
        0.0173162311086590,
        0.0173162311086590,
        0.0173162311086590,
        0.0173162311086590,
        # Weights for number 37
        0.0525209234008020,
        0.0112801452093300,
        0.0314235183624540,
        0.0470725025041940,
        0.0473635865363550,
        0.0311675290457940,
        0.0079757714650740,
        0.0368484027287320,
        0.0174014633038220,
        0.0155217868390450,
        0.0112801452093300,
        0.0112801452093300,
        0.0314235183624540,
        0.0314235183624540,
        0.0470725025041940,
        0.0470725025041940,
        0.0473635865363550,
        0.0473635865363550,
        0.0311675290457940,
        0.0311675290457940,
        0.0079757714650740,
        0.0079757714650740,
        0.0368484027287320,
        0.0368484027287320,
        0.0368484027287320,
        0.0368484027287320,
        0.0368484027287320,
        0.0174014633038220,
        0.0174014633038220,
        0.0174014633038220,
        0.0174014633038220,
        0.0174014633038220,
        0.0155217868390450,
        0.0155217868390450,
        0.0155217868390450,
        0.0155217868390450,
        0.0155217868390450,
        # Weights for number 42
        0.0218835813694290,
        0.0327883535441250,
        0.0517741045072920,
        0.0421625887369930,
        0.0144336996697770,
        0.0049234036024000,
        0.0246657532125640,
        0.0385715107870610,
        0.0144363081135340,
        0.0050102288385010,
        0.0218835813694290,
        0.0218835813694290,
        0.0327883535441250,
        0.0327883535441250,
        0.0517741045072920,
        0.0517741045072920,
        0.0421625887369930,
        0.0421625887369930,
        0.0144336996697770,
        0.0144336996697770,
        0.0049234036024000,
        0.0049234036024000,
        0.0246657532125640,
        0.0246657532125640,
        0.0246657532125640,
        0.0246657532125640,
        0.0246657532125640,
        0.0385715107870610,
        0.0385715107870610,
        0.0385715107870610,
        0.0385715107870610,
        0.0385715107870610,
        0.0144363081135340,
        0.0144363081135340,
        0.0144363081135340,
        0.0144363081135340,
        0.0144363081135340,
        0.0050102288385010,
        0.0050102288385010,
        0.0050102288385010,
        0.0050102288385010,
        0.0050102288385010,
        # Weights for number 48
        0.0019168756428490,
        0.0442490272711450,
        0.0511865487188520,
        0.0236877358706880,
        0.0132897756900210,
        0.0047489166081920,
        0.0385500725995930,
        0.0272158143206240,
        0.0021820773667970,
        0.0215053198477310,
        0.0076739426310490,
        0.0019168756428490,
        0.0019168756428490,
        0.0442490272711450,
        0.0442490272711450,
        0.0511865487188520,
        0.0511865487188520,
        0.0236877358706880,
        0.0236877358706880,
        0.0132897756900210,
        0.0132897756900210,
        0.0047489166081920,
        0.0047489166081920,
        0.0385500725995930,
        0.0385500725995930,
        0.0385500725995930,
        0.0385500725995930,
        0.0385500725995930,
        0.0272158143206240,
        0.0272158143206240,
        0.0272158143206240,
        0.0272158143206240,
        0.0272158143206240,
        0.0021820773667970,
        0.0021820773667970,
        0.0021820773667970,
        0.0021820773667970,
        0.0021820773667970,
        0.0215053198477310,
        0.0215053198477310,
        0.0215053198477310,
        0.0215053198477310,
        0.0215053198477310,
        0.0076739426310490,
        0.0076739426310490,
        0.0076739426310490,
        0.0076739426310490,
        0.0076739426310490,
        # Weights for number 52
        0.0468756974276420,
        0.0064058785785850,
        0.0417102967393870,
        0.0268914842500640,
        0.0421325227616500,
        0.0300002668427730,
        0.0142000989250240,
        0.0035824623512730,
        0.0327731474606270,
        0.0152983062484410,
        0.0023862441928390,
        0.0190847927558990,
        0.0068500545465420,
        0.0064058785785850,
        0.0064058785785850,
        0.0417102967393870,
        0.0417102967393870,
        0.0268914842500640,
        0.0268914842500640,
        0.0421325227616500,
        0.0421325227616500,
        0.0300002668427730,
        0.0300002668427730,
        0.0142000989250240,
        0.0142000989250240,
        0.0035824623512730,
        0.0035824623512730,
        0.0327731474606270,
        0.0327731474606270,
        0.0327731474606270,
        0.0327731474606270,
        0.0327731474606270,
        0.0152983062484410,
        0.0152983062484410,
        0.0152983062484410,
        0.0152983062484410,
        0.0152983062484410,
        0.0023862441928390,
        0.0023862441928390,
        0.0023862441928390,
        0.0023862441928390,
        0.0023862441928390,
        0.0190847927558990,
        0.0190847927558990,
        0.0190847927558990,
        0.0190847927558990,
        0.0190847927558990,
        0.0068500545465420,
        0.0068500545465420,
        0.0068500545465420,
        0.0068500545465420,
        0.0068500545465420,
        # Weights for number 61
        0.0334371992908030,
        0.0050934154405070,
        0.0146708645276380,
        0.0243508783536720,
        0.0311075508689690,
        0.0312571112186200,
        0.0248156543396650,
        0.0140560730705570,
        0.0031946761737790,
        0.0081196553189930,
        0.0268057422831630,
        0.0184599932108220,
        0.0084768685343280,
        0.0182927967700250,
        0.0066656320041650,
        0.0050934154405070,
        0.0050934154405070,
        0.0146708645276380,
        0.0146708645276380,
        0.0243508783536720,
        0.0243508783536720,
        0.0311075508689690,
        0.0311075508689690,
        0.0312571112186200,
        0.0312571112186200,
        0.0248156543396650,
        0.0248156543396650,
        0.0140560730705570,
        0.0140560730705570,
        0.0031946761737790,
        0.0031946761737790,
        0.0081196553189930,
        0.0081196553189930,
        0.0081196553189930,
        0.0081196553189930,
        0.0081196553189930,
        0.0268057422831630,
        0.0268057422831630,
        0.0268057422831630,
        0.0268057422831630,
        0.0268057422831630,
        0.0184599932108220,
        0.0184599932108220,
        0.0184599932108220,
        0.0184599932108220,
        0.0184599932108220,
        0.0084768685343280,
        0.0084768685343280,
        0.0084768685343280,
        0.0084768685343280,
        0.0084768685343280,
        0.0182927967700250,
        0.0182927967700250,
        0.0182927967700250,
        0.0182927967700250,
        0.0182927967700250,
        0.0066656320041650,
        0.0066656320041650,
        0.0066656320041650,
        0.0066656320041650,
        0.0066656320041650,
        # Weights for number 70
        0.0308099399376470,
        0.0090724366794040,
        0.0187613169395940,
        0.0194410979854770,
        0.0277539486108100,
        0.0322562253514570,
        0.0250740326169220,
        0.0152719279718320,
        0.0067939220229630,
        -0.0022230987299200,
        0.0063319140764060,
        0.0272575380491380,
        0.0176767856494650,
        0.0183794846380700,
        0.0081047328081920,
        0.0076341290707250,
        0.0000461876607940,
        0.0090724366794040,
        0.0090724366794040,
        0.0187613169395940,
        0.0187613169395940,
        0.0194410979854770,
        0.0194410979854770,
        0.0277539486108100,
        0.0277539486108100,
        0.0322562253514570,
        0.0322562253514570,
        0.0250740326169220,
        0.0250740326169220,
        0.0152719279718320,
        0.0152719279718320,
        0.0067939220229630,
        0.0067939220229630,
        -0.0022230987299200,
        -0.0022230987299200,
        0.0063319140764060,
        0.0063319140764060,
        0.0063319140764060,
        0.0063319140764060,
        0.0063319140764060,
        0.0272575380491380,
        0.0272575380491380,
        0.0272575380491380,
        0.0272575380491380,
        0.0272575380491380,
        0.0176767856494650,
        0.0176767856494650,
        0.0176767856494650,
        0.0176767856494650,
        0.0176767856494650,
        0.0183794846380700,
        0.0183794846380700,
        0.0183794846380700,
        0.0183794846380700,
        0.0183794846380700,
        0.0081047328081920,
        0.0081047328081920,
        0.0081047328081920,
        0.0081047328081920,
        0.0081047328081920,
        0.0076341290707250,
        0.0076341290707250,
        0.0076341290707250,
        0.0076341290707250,
        0.0076341290707250,
        0.0000461876607940,
        0.0000461876607940,
        0.0000461876607940,
        0.0000461876607940,
        0.0000461876607940,
        # Weights for number 73
        0.0329063313889190,
        0.0103307318912720,
        0.0223872472630160,
        0.0302661258694680,
        0.0304909678021980,
        0.0241592127416410,
        0.0160508035868010,
        0.0080845802617840,
        0.0020793620274850,
        0.0038848769049810,
        0.0255741606120220,
        0.0088809035733380,
        0.0161245467617310,
        0.0024919418174910,
        0.0182428401189510,
        0.0102585637361990,
        0.0037999288553020,
        0.0103307318912720,
        0.0103307318912720,
        0.0223872472630160,
        0.0223872472630160,
        0.0302661258694680,
        0.0302661258694680,
        0.0304909678021980,
        0.0304909678021980,
        0.0241592127416410,
        0.0241592127416410,
        0.0160508035868010,
        0.0160508035868010,
        0.0080845802617840,
        0.0080845802617840,
        0.0020793620274850,
        0.0020793620274850,
        0.0038848769049810,
        0.0038848769049810,
        0.0038848769049810,
        0.0038848769049810,
        0.0038848769049810,
        0.0255741606120220,
        0.0255741606120220,
        0.0255741606120220,
        0.0255741606120220,
        0.0255741606120220,
        0.0088809035733380,
        0.0088809035733380,
        0.0088809035733380,
        0.0088809035733380,
        0.0088809035733380,
        0.0161245467617310,
        0.0161245467617310,
        0.0161245467617310,
        0.0161245467617310,
        0.0161245467617310,
        0.0024919418174910,
        0.0024919418174910,
        0.0024919418174910,
        0.0024919418174910,
        0.0024919418174910,
        0.0182428401189510,
        0.0182428401189510,
        0.0182428401189510,
        0.0182428401189510,
        0.0182428401189510,
        0.0102585637361990,
        0.0102585637361990,
        0.0102585637361990,
        0.0102585637361990,
        0.0102585637361990,
        0.0037999288553020,
        0.0037999288553020,
        0.0037999288553020,
        0.0037999288553020,
        0.0037999288553020,
        # Weights for number 79
        0.0330570555416240,
        0.0008670191856630,
        0.0116600527164480,
        0.0228769363564210,
        0.0304489826739380,
        0.0306248917253550,
        0.0243680576768000,
        0.0159974320320240,
        0.0076983018156020,
        -0.0006320604974880,
        0.0017511343011930,
        0.0164658391895760,
        0.0048390335404850,
        0.0258049065346500,
        0.0084710910544410,
        0.0183549141062800,
        0.0007044046779080,
        0.0101126849274620,
        0.0035739093859500,
        0.0008670191856630,
        0.0008670191856630,
        0.0116600527164480,
        0.0116600527164480,
        0.0228769363564210,
        0.0228769363564210,
        0.0304489826739380,
        0.0304489826739380,
        0.0306248917253550,
        0.0306248917253550,
        0.0243680576768000,
        0.0243680576768000,
        0.0159974320320240,
        0.0159974320320240,
        0.0076983018156020,
        0.0076983018156020,
        -0.0006320604974880,
        -0.0006320604974880,
        0.0017511343011930,
        0.0017511343011930,
        0.0164658391895760,
        0.0164658391895760,
        0.0164658391895760,
        0.0164658391895760,
        0.0164658391895760,
        0.0048390335404850,
        0.0048390335404850,
        0.0048390335404850,
        0.0048390335404850,
        0.0048390335404850,
        0.0258049065346500,
        0.0258049065346500,
        0.0258049065346500,
        0.0258049065346500,
        0.0258049065346500,
        0.0084710910544410,
        0.0084710910544410,
        0.0084710910544410,
        0.0084710910544410,
        0.0084710910544410,
        0.0183549141062800,
        0.0183549141062800,
        0.0183549141062800,
        0.0183549141062800,
        0.0183549141062800,
        0.0007044046779080,
        0.0007044046779080,
        0.0007044046779080,
        0.0007044046779080,
        0.0007044046779080,
        0.0101126849274620,
        0.0101126849274620,
        0.0101126849274620,
        0.0101126849274620,
        0.0101126849274620,
        0.0035739093859500,
        0.0035739093859500,
        0.0035739093859500,
        0.0035739093859500,
        0.0035739093859500,
    ]
)


def rule(order):
    """
    Return the symmetric Gauss quadrature of given order.

    The order must be between 1 and 20.
    """
    if order < 1 or order > 20:
        raise ValueError(f"Symmetric Gauss quadrature order must be between 1 and 20. Provided: {order}")
    npoints = points_per_order[order - 1]
    address = points_address[npoints - 1]
    bary_coords = coords[3 * address : 3 * (address + npoints)].reshape((3, npoints), order="F")
    points = _np.asfortranarray(_np.vstack([bary_coords[1, :], bary_coords[2, :]]))
    return (points, 0.5 * weights[address : address + npoints])


def get_number_of_quad_points(order):
    """Get number of quad points per order."""
    return points_per_order[order - 1]
